@@ -52,7 +52,15 @@ case $# in
      ;;
 esac
 
+maybe_activate_venv() {
+  if [ ! -z "${NANOARROW_PYTHON_VENV}" ]; then
+    source "${NANOARROW_PYTHON_VENV}/bin/activate"
+  fi
+}
+
 function main() {
+    maybe_activate_venv
+
     SANDBOX_DIR="${TARGET_NANOARROW_DIR}/_coverage"
     if [ -d "${SANDBOX_DIR}" ]; then
         rm -rf "${SANDBOX_DIR}"
@@ -67,7 +75,7 @@ function main() {
     cmake "${TARGET_NANOARROW_DIR}" \
         -DNANOARROW_BUILD_TESTS=ON -DNANOARROW_CODE_COVERAGE=ON
     cmake --build .
-    ctest .
+    CTEST_OUTPUT_ON_FAILURE=1 ctest .
 
     popd
 
@@ -79,7 +87,7 @@ function main() {
     cmake "${TARGET_NANOARROW_DIR}/extensions/nanoarrow_ipc" \
         -DNANOARROW_IPC_BUILD_TESTS=ON -DNANOARROW_IPC_CODE_COVERAGE=ON
     cmake --build .
-    ctest .
+    CTEST_OUTPUT_ON_FAILURE=1 ctest .
 
     popd
 
@@ -93,7 +101,7 @@ function main() {
     cmake "${TARGET_NANOARROW_DIR}/extensions/nanoarrow_device" \
         -DNANOARROW_DEVICE_BUILD_TESTS=ON -DNANOARROW_DEVICE_CODE_COVERAGE=ON
     cmake --build .
-    ctest .
+    CTEST_OUTPUT_ON_FAILURE=1 ctest .
 
     popd
 
@@ -107,6 +115,7 @@ function main() {
         --exclude "*/gtest/*" \
         --exclude "*/flatcc/*" \
         --exclude "*_generated.h" \
+        --exclude "*nanoarrow/_deps/*" \
         --output-file coverage.info
 
     # Generate the html coverage while we're here
@@ -137,6 +146,35 @@ function main() {
     show_header "R package coverage summary"
     Rscript -e 'library(covr); print(readRDS("r_coverage.rds"))'
     popd
+
+    # Build + test Python package with cython/gcc coverage options
+    show_header "Build + test Python package"
+    pushd "${SANDBOX_DIR}"
+    TARGET_NANOARROW_PYTHON_DIR="${TARGET_NANOARROW_DIR}/python"
+
+    pushd "${TARGET_NANOARROW_PYTHON_DIR}"
+    NANOARROW_PYTHON_COVERAGE=1 python -m pip install -e .
+
+    # Run tests + coverage.py (generates .coverage with absolute file paths)
+    python -m pytest --cov ./src/nanoarrow
+
+    # Generate HTML report (file paths not important since it's just for viewing)
+    python -m coverage html
+    mv htmlcov "${SANDBOX_DIR}/python_htmlcov"
+
+    # Move .coverage to the root directory and generate coverage.xml
+    # (generates relative file paths from the root of the repo)
+    mv .coverage ..
+    cp .coveragerc ..
+    pushd ..
+    python -m coverage xml
+    mv coverage.xml "${SANDBOX_DIR}/python_coverage.xml"
+    mv .coverage "${SANDBOX_DIR}/python_coverage.db"
+    rm .coveragerc
+    popd
+
+    popd
+    popd
 }
 
 main
